//import java.awt.Dimension;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

public class MVCDemo implements Runnable {

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new MVCDemo());
	}

	private ProportionModel propModel = new MyProportionModel();
	
	
	@Override
	public void run() {
		JFrame frame = new JFrame("MVCDemo");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setLayout(new FlowLayout());
		
		PiechartProportionView piechart = new PiechartProportionView(propModel);
		piechart.setPreferredSize(new Dimension(300,300));
		propModel.addPropertyChangeListener(piechart);
		frame.add(piechart);
		
		PercentageProportionController percentage = new PercentageProportionController(propModel);
		percentage.addActionListener(percentage);
		frame.add(percentage);
		
		frame.pack();
		frame.setVisible(true);
	}

	
	public static class PiechartProportionView extends JPanel implements PropertyChangeListener {
		private static final long serialVersionUID = 1L;

		private ProportionModel model;
		
		public PiechartProportionView(ProportionModel model) {
			super();
			this.model = model;
		}

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			repaint();
		}
		
		@Override
		public void paintComponent(Graphics g) {
			int w = getWidth();
			int h = getHeight();
			int diam = Math.min(h, w);
			int x = (w - diam)/2;
			int y = (h - diam)/2;
			g.drawOval(x, y, diam, diam);
			int partFilled = (int)Math.round(model.getProportion() * 360.0);
			g.fillArc(x, y, diam, diam, 90, -partFilled);
		}
	}
	
	public static class PercentageProportionController extends JTextField implements ActionListener {
		private static final long serialVersionUID = 1L;
		
		private ProportionModel model;
		
		public PercentageProportionController(ProportionModel model) {
			super("0", 10);
			this.model = model;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			model.setProportion(Double.parseDouble(getText()) / 100.0);
		}
		
	}
	
}
